@@ -9,6 +9,11 @@ arg_values = [("blob", None), ("blob", 5), ("shitomasi", None), ("shitomasi", 5)
 
 @pytest.mark.parametrize(arg_names, arg_values)
 def test_feature(method, max_num_features):
+    if method == "blob":
+        pytest.importorskip("skimage")
+    if method == "shitomasi":
+        pytest.importorskip("cv2")
+
     input_field, _ = get_precipitation_fields(0, 0, True, True, None, "mch")
 
     detector = feature.get_method(method)
